package com.fastbee.project.service.impl;

import cn.hutool.core.codec.Base64;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.fastbee.common.constant.ProjectLevelConstant;
import com.fastbee.common.core.domain.entity.SysDept;
import com.fastbee.common.exception.ServiceException;
import com.fastbee.common.holder.ProjectHolder;
import com.fastbee.common.utils.DateUtils;
import com.fastbee.project.domain.Project;
import com.fastbee.project.mapper.ProjectMapper;
import com.fastbee.project.service.IProjectService;

import com.fastbee.system.mapper.SysDeptMapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 项目Service业务层处理
 *
 * @author kerwincui
 * &#064;date  2024-09-26
 */
@Service
public class ProjectServiceImpl implements IProjectService
{

        private final ProjectMapper projectMapper;


        private final SysDeptMapper sysDeptMapper;

        public ProjectServiceImpl(ProjectMapper projectMapper, SysDeptMapper sysDeptMapper) {
                this.projectMapper = projectMapper;
                this.sysDeptMapper = sysDeptMapper;
        }

        /**
         * 查询项目
         *
         * @param projectId 项目主键
         * @return 项目
         */
        @Override
        public Project selectProjectByProjectId(Long projectId)
        {
                return projectMapper.selectProjectByProjectId(projectId);
        }

        /**
         * 查询项目列表
         */
        @Override
        public List<Project> selectProjectList(Project project)
        {
                return  new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getProjectId, Project::getProjectName, Project::getScope,
                                Project::getAdministrativeArea, Project::getDeptId,Project::getAdministrativeAreaCode,
                                Project::getLogo, Project::getImage, Project::getLevel, Project::getDeptName,
                                Project::getCreateTime)
                        .like(StringUtils.isNotBlank(project.getSearchValue()),Project::getProjectName, project.getSearchValue())
                        .list();
        }

        /**
         * 新增项目
         */
        @Override
        public int insertProject(Project project)
        {       //判断选择的机构下是否已经绑定项目
                List<Project> projectList = new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getProjectId, Project::getProjectName)
                        .eq(Project::getDeptId, project.getDeptId()).list();
                if(!projectList.isEmpty()){
                        throw new ServiceException("该机构下已经存在项目");
                }
                project.setCreateTime(DateUtils.getNowDate());
                return projectMapper.insertProject(project);
        }

        /**
         * 修改项目
         * @return 结果
         */
        @Override
        public int updateProject(Project project)
        {
                //判断选择的机构下是否已经绑定项目
                Project one = new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getProjectId, Project::getProjectName,Project::getDeptId)
                        .eq(Project::getDeptId, project.getDeptId()).one();
                if(Objects.isNull(one)){
                        throw new ServiceException("项目不存在！");
                }
                if(!one.getDeptId().equals(project.getDeptId())){
                        throw new ServiceException("非法修改项目机构！");
                }
                project.setUpdateTime(DateUtils.getNowDate());
                return projectMapper.updateProject(project);
        }

        /**
         * 批量删除项目
         */
        @Override
        public int deleteProjectByProjectIds(Long[] projectIds)
        {
                return projectMapper.deleteProjectByProjectIds(projectIds);
        }


        /**
         * 获取管理员所管理的项目id以及子项目id列表
         */
        @Override
        public Map<String, Object> selectProjectByUserId(Long userId) {
                //查询该管理员所管理的项目id以及子项目id列表
                Project project = new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getProjectId, Project::getProjectName,Project::getLevel,Project::getDeptId)
                        .eq(Project::getOwnerId, userId)
                        .one();
                if(Objects.isNull(project)){
                        return null;
                }
                List<Long> projects = getAllProjects(project.getProjectId());
                //把列表进行Base64编码
                String projectsBase64 = Base64.encode(projects.toString());
                HashMap<String , Object> map = new HashMap<>();
                map.put("projectIds",projectsBase64);
                map.put("projectId",project.getProjectId());
                map.put("projectName", project.getProjectName());
                map.put("projectLevel",project.getLevel().equals("市级")? ProjectLevelConstant.LEVEL_CITY :ProjectLevelConstant.LEVEL_COUNTY);
                return map;
        }

        /**
         * 查询未绑定项目的部门列表
         */
        @Override
        public List<SysDept> getUnbindDeptList() {
                //查询所有机构
                List<SysDept> sysDeptList = new LambdaQueryChainWrapper<>(sysDeptMapper)
                        .select(SysDept::getDeptId, SysDept::getDeptName)
                        .list();
                //查询所有项目
                List<Project> projectList = new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getProjectId, Project::getDeptId)
                        .list();
                //遍历项目，把已绑定项目的机构从机构列表中移除
                for (Project project : projectList) {
                        if (Objects.nonNull(project.getDeptId()))
                                sysDeptList.removeIf(sysDept -> sysDept.getDeptId().equals(project.getDeptId()));
                }
                return sysDeptList;
        }

        /**
         * 查询管理员可管理项目下拉框列表
         */
        @Override
        public List<Project> selectAdminProjectList() {
                //判断管理员级别
                List<Project> projectList=new ArrayList<>();
                //超级管理员
                if(Objects.isNull(ProjectHolder.getProjectInfo().getProjectIdList())){
                        //查询所有一级项目列表
                        projectList = new LambdaQueryChainWrapper<>(projectMapper)
                                .select(Project::getProjectId, Project::getProjectName)
                                .isNull(Project::getParentId)
                                .list();
                }
                //市级/县级项目
                else if(Objects.nonNull(ProjectHolder.getProjectInfo().getProjectIdList())){
                        //查询所有子一级项目列表
                        projectList = new LambdaQueryChainWrapper<>(projectMapper)
                                .select(Project::getProjectId, Project::getProjectName)
                                .in(Project::getParentId, ProjectHolder.getProjectInfo().getProjectIdList())
                                .list();
                }
                return projectList;
        }

        public List<Long> getAllProjects(Long projectId) {
                List<Long> allProjects = new ArrayList<>();

                // 查询当前项目
                Project currentProject = new LambdaQueryChainWrapper<>(projectMapper)
                        .select(Project::getProjectId, Project::getProjectName)
                        .eq(Project::getProjectId, projectId)
                        .one();

                if (currentProject != null) {
                        allProjects.add(currentProject.getProjectId());

                        // 查询当前项目的子项目
                        List<Project> children = new LambdaQueryChainWrapper<>(projectMapper)
                                .select(Project::getProjectId, Project::getProjectName)
                                .eq(Project::getParentId, projectId)
                                .list();

                        // 递归查询子项目
                        for (Project child : children) {
                                allProjects.addAll(getAllProjects(child.getProjectId()));
                        }
                }

                return allProjects;
        }
}

